package cn.fw.morax.domain.dto;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.EvalIndicator;
import cn.fw.morax.domain.enums.DataTypeEnum;
import cn.fw.morax.domain.enums.EvalRuleTypeEnum;
import cn.fw.morax.domain.enums.MonthDayTypeEnum;
import cn.fw.morax.domain.enums.TargetTypeEnum;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.util.List;

/**
 * @author : kurisu
 * @date : 2022-4-2
 * @desc : 指标库
 */
@Data
public class EvalIndicatorDTO {

    private Long id;

    /**
     * 指标名称
     */
    @NotBlank(message = "指标名称不能为空")
    @Length(max = 255, message = "指标名称超出最大长度限制")
    private String name;

    /**
     * 归属系统id
     */
    @NotNull(message = "归属系统不能为空")
    private Long sysId;

    /**
     * 归属系统名称
     */
    @NotNull(message = "归属系统名称不能为空")
    private String sysName;

    /**
     * 数据类型；1:台数 2:百分比 3:金额
     */
    @NotNull(message = "数据类型不能为空")
    private DataTypeEnum dataType;

    /**
     * 绩效目标值类型; 1:无 2:百分比 3:金额
     */
    @NotNull(message = "绩效目标值类型不能为空")
    private TargetTypeEnum targetType;

    /**
     * 导入操作角色编码
     */
    @NotBlank(message = "导入操作角色编码不能为空")
    private String roleCode;

    /**
     * 导入操作角色名称
     */
    @NotBlank(message = "导入操作角色名称不能为空")
    private String roleName;

    /**
     * 频率类型; 1:按滚动天 2:按星期 3:按月指定日期 4：按滚动月
     */
    @NotNull(message = "频率类型不能为空")
    private EvalRuleTypeEnum ruleType;

    /**
     * 频率值 ,号分隔（按星期、按月指定日期使用）
     */
    private List<Integer> ruleValues;

    /**
     * 滚动月数/滚动天数
     */
    private Integer rollValue;

    /**
     * 开始日期
     */
    private LocalDate startTime;

    /**
     * 月天; 1:月第几天 2:月倒数第几天
     */
    private MonthDayTypeEnum monthDayType;

    /**
     * 月天数
     */
    private Integer monthDay;

    /**
     * 转换为db对象
     *
     * 1. 滚动天数		rule_values（滚动天数），start_time（开始日期）
     * 2. 按星期		    rule_values（星期）
     * 3. 按月指定日期	rule_values（滚动月数、滚动开始日）
     * 4. 按滚动月		rule_values（滚动月数、滚动开始日），start_time（开始日期），
     * 				    order_type（按滚动月计算顺序）
     * @return
     */
    public EvalIndicator convertToDB(){
        EvalIndicator indicators = EvalIndicator.builder()
                .name(this.getName())
                .enable(Boolean.FALSE)
//                .dataType(this.getDataType())
//                .targetType(this.targetType)
                .sysId(this.getSysId())
                .sysName(this.getSysName())
                .roleCode(this.getRoleCode())
                .roleName(this.getRoleName())
                .build();
        if (PublicUtil.isEmpty(this.getId())) {
            indicators.setId(this.getId());
        }
        return indicators;
    }


}