package cn.fw.morax.domain.dto.eval;

import cn.fw.common.web.annotation.LoginContextField;
import cn.fw.morax.domain.db.eval.EvalGroup;
import cn.fw.morax.domain.enums.EvalTypeEnum;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.Future;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 考评组
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalDTO {

    /**
     * 考评组id（前一次）
     */
    private Long id;

    /**
     * 考评，活动名称
     */
    @NotBlank(message = "名称不能为空")
    @Length(max = 255, message = "名称超出最大限制")
    private String name;

    /**
     * 考评类型; 1:活动考评 2:其他考评
     */
    @NotNull(message = "考评类型不能为空")
    private EvalTypeEnum type;

    /**
     * 集团id
     force     */
    @LoginContextField(LoginContextField.Name.GROUP_ID)
    private Long groupId;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_NAME)
    private String userName;

    /**
     * 用户名
     force     */
    @LoginContextField(LoginContextField.Name.USER_ID)
    private Long userId;

    /**
     * 考评组
     */
    @NotNull(message = "考评组不能为空")
    @Size(min = 1, message = "考评组不能为空")
    @Valid
    private List<EvalGroupDTO> evalGroups;


}
