package cn.fw.morax.domain.dto.eval;

import cn.fw.common.data.entity.BaseAuditableTimeEntity;
import cn.fw.morax.domain.db.eval.EvalGroupRewardParam;
import cn.fw.morax.domain.enums.EvalRewardCalMethodEnum;
import cn.fw.morax.domain.enums.RankTypeEnum;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 考评奖惩
 * </p>
 *
 * @author jiangchao
 * @since 2022-12-09
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class EvalGroupRewardDTO {

    private static final long serialVersionUID = 1L;

    /**
     * 考评奖惩指标名称
     */
    @NotBlank(message = "考评奖惩指标名称不能为空")
    @Length(max = 255, message = "考评奖惩指标名称超出最大限制")
    private String name;

    /**
     * 计算方式; 1: 排名正负激励 2:阶梯奖励(翻倍) 3:总金额X达成率占比 4:固定值X达成率 5: 固定值+超额部分追加奖励
     */
    @NotNull(message = "计算方式不能为空")
    private EvalRewardCalMethodEnum calMethod;

    /**
     * 排名方式;1:排名序号 2:排名人数百分比
     */
    private RankTypeEnum rankType;

    /**
     * 金额
     */
//    @NotNull(message = "金额不能为空")
    private BigDecimal money;

    /**
     * 追加奖励 XX 元/台，超额奖励用
     */
    private BigDecimal extraMoney;

    /**
     * 提成参数
     */
    @Valid
    private List<EvalGroupRewardParamDTO> commissionParams;

    /**
     * 台阶参数
     */
    @Valid
    private List<EvalGroupRewardParamDTO> ladderParams;

    /**
     * 阶梯
     */
    @Valid
    private List<EvalGroupRewardLaddersDTO> ladders;

    /**
     * 条件
     */
    @Valid
    private List<EvalGroupRewardPreconditionDTO> conds;

}
