package cn.fw.morax.server.controller.erp;


import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.EvalIndicatorDTO;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.kpi.IndicatorAbleDTO;
import cn.fw.morax.domain.dto.query.EvalIndicatorQueryDTO;
import cn.fw.morax.domain.vo.eval.EvalIndicatorVO;
import cn.fw.morax.domain.vo.eval.EvalVO;
import cn.fw.morax.service.biz.eval.EvalGroupBizService;
import cn.fw.morax.service.biz.eval.EvalGroupDataService;
import cn.fw.morax.service.biz.eval.EvalIndicatorBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评指标
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/erp/eval")
public class EvalIndicatorController {

    private final EvalGroupBizService evalGroupBizService;
    private final EvalGroupDataService evalGroupDataService;
    private final EvalIndicatorBizService evalIndicatorBizService;

    /**
     * 考评指标列表
     *
     * @param dto
     * @return
     */
    @GetMapping("/indicator/page")
    @ControllerMethod("考评指标列表")
    public Message<AppPage<EvalIndicatorVO>> evalIndicatorPage(@Validated EvalIndicatorQueryDTO dto) {
        return success(evalIndicatorBizService.evalIndicatorPage(dto));
    }

    /**
     * 考评指标详情
     *
     * @return
     */
    @GetMapping("/indicator")
    @ControllerMethod("考评指标详情")
    public Message<EvalIndicatorVO> evalIndicatorDetail(@NotNull(message = "考评指标id不能为空") Long id) {
        return success(evalIndicatorBizService.evalIndicatorDetail(id));
    }

    /**
     * 考评指标禁用启用
     *
     * @return
     */
    @PutMapping("/indicator/enable")
    @ControllerMethod("考评指标禁用启用")
    public Message<Void> enableIndicator(@Validated @RequestBody IndicatorAbleDTO dto) {
        evalIndicatorBizService.enableIndicator(dto.getIndicatorId(), dto.getEnable());
        return success();
    }

    /**
     * 保存考评指标
     *
     * @param dto 考评指标
     * @return
     */
    @PostMapping("/indicator")
    @ControllerMethod("保存考评指标")
    public Message<Void> saveEvalIndicator(@Validated @RequestBody EvalIndicatorDTO dto,
                                                                @CurrentUser LoginAuthBean user) {
        evalIndicatorBizService.saveEvalIndicator(dto, user);
        return success();
    }

    /**
     * 保存考评
     *
     * @param dto 考评
     * @return
     */
    @PostMapping("/group")
    @ControllerMethod("保存考评")
    public Message<Void> saveEvalGroup(@Validated @RequestBody EvalDTO dto,
                                       @CurrentUser LoginAuthBean user) {
        evalGroupBizService.saveEvalGroup(dto, user);
        return success();
    }

    /**
     * 考评详情
     *
     * @param id
     * @return
     */
    @GetMapping("/group")
    @ControllerMethod("考评详情")
    public Message<EvalVO> evalGroupDetail(@NotNull(message = "考评组id不能为空") Long id) {

        return success(evalGroupBizService.evalDetail(id));
    }

    /**
     * 考评删除
     *
     * @return
     */
    @GetMapping("/group/del")
    @ControllerMethod("考评删除")
    public Message<Void> delEval(@NotNull(message = "考评id不能为空") Long id) {
        evalGroupDataService.delEval(id);
        return success();
    }

    /**
     * 保存考评草稿
     *
     * @return
     */
    @PostMapping("/group/draft")
    @ControllerMethod("保存考评草稿")
    public Message<Void> saveEvalDraft(@RequestBody EvalDTO dto) {
        evalGroupDataService.saveEvalDraft(dto);
        return success();
    }

    /**
     * 获取考评草稿
     *
     * @return
     */
    @GetMapping("/group/draft")
    @ControllerMethod("获取考评草稿")
    public Message<EvalDTO> getEvalDraft(@NotNull(message = "考评id不能为空") Long id) {
        return success(evalGroupDataService.getEvalDraft(id));
    }

//    /**
//     * 实时查询绩效组人员
//     *
//     * @param dto
//     * @return
//     */
//    @GetMapping("/real-time/staffs")
//    @ControllerMethod("实时查询岗位适用人员")
//    public Message<List<KpiGroupUserVO>> queryShopPostRealTimeStaffs(@Validated KpiGroupStaffQueryDTO dto) {
//        return success(kpiGroupUserBizService.queryShopPostRealTimeStaffs(dto));
//    }

}
