package cn.fw.morax.service.biz.eval;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.dto.eval.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.service.biz.ApprovalBizService;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.eval.*;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@Service
@Slf4j
@RequiredArgsConstructor
public class EvalGroupBizService {

    private final EvalGroupIndicatorPreconditionLaddersService evalGroupIndicatorPreconditionLaddersService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupRewardLaddersService evalGroupRewardLaddersService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalIndicatorService evalIndicatorService;
    private final EvalGroupDataService evalGroupDataService;
    private final ApprovalRecordService approvalRecordService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final EvalGroupService evalGroupService;
    private final EvalService evalService;
    private final FlowApproveRpc flowApproveRpc;
    private final ApprovalBizService approvalBizService;
    private final FlowVal flowVal;

    @Value("${spring.cache.custom.global-prefix}:eval:save:")
    @Getter
    private String saveGroupPrefix;


    /**
     * 考评组保存
     *
     * @param dto
     * @param user
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSaveGroupPrefix()", key = "#dto.getPostId()", message = "保存中，请勿重复操作")
    public void saveEvalGroup(EvalDTO dto, LoginAuthBean user) {

        for (EvalGroupDTO evalGroupDTO : dto.getEvalGroups()) {
            checkTime(evalGroupDTO);
            checkNameRepetition(dto.getId(), dto.getName());
            checkIndicator(evalGroupDTO);
            checkReward(evalGroupDTO);
            checkEvalIndicatorLadders(evalGroupDTO.getIndicators());
            checkEvalRewardLadders(evalGroupDTO.getRewards());
        }

        Eval eval = evalGroupDataService.saveEval(dto);
        for (EvalGroupDTO evalGroupDTO : dto.getEvalGroups()) {
            EvalGroup evalGroup = evalGroupDataService.saveEvalGroup(evalGroupDTO, eval);
            evalGroupDataService.saveEvalGroupIndicator(evalGroup, evalGroupDTO.getIndicators());
            evalGroupDataService.saveEvalGroupReward(evalGroup, evalGroupDTO.getRewards());
//            Integer staffNum = kpiGroupUserBizService.queryShopPostCurStaffNum(evalGroupDTO.getPostId(), evalGroupDTO.getShopIds());
        }
        approvalBizService.applyApproveEval(eval, dto.getEvalGroups(), user);
        //保存草稿
        evalGroupDataService. saveSettingDraft(dto, eval);
    }

    /**
     * 检查时间
     *
     * @param dto
     */
    public void checkTime(EvalGroupDTO dto) {
        BV.isTrue(dto.getBeginTime().isBefore(dto.getOverTime()), "考评组【" + dto.getName()+ "】生效时间必须在结束时间之前");
    }

    /**
     * 检查绩效组排名名称是否重复
     *
     * @param id
     * @param name
     */
    public void checkNameRepetition(Long id, String name) {
        int count = evalGroupService.count(Wrappers.<EvalGroup>lambdaQuery()
                .eq(EvalGroup::getName, name)
                .ne(PublicUtil.isNotEmpty(id), EvalGroup::getId, id)
                .eq(EvalGroup::getYn, Boolean.TRUE)
//                .eq(EvalGroup::getStatus, SettingStatusEnum.EFFECTIVE)
        );
        BV.isTrue(count <= 0, "绩效组名称重复，请重新输入");
    }

    /**
     * 检查阶梯分值
     * @param indicators
     */
    private void checkEvalIndicatorLadders(List<EvalGroupIndicatorDTO> indicators) {
        Integer baseScore = null;
        for (EvalGroupIndicatorDTO indicatorDto : indicators) {
            if (ScoreWayEnum.NORMAL.equals(indicatorDto.getScoreWay())) {
                continue;
            }

            baseScore = indicatorDto.getBaseScore();
            List<EvalGroupIndicatorLaddersDTO> ladders = indicatorDto.getLadders();
            Collections.sort(ladders);

            //阶梯值校验  标准分不能超过指标基础分，阶梯值必须连续
            for (EvalGroupIndicatorLaddersDTO laddersDto : ladders) {
                //阶梯标准分不能超过绩效分值
                if (laddersDto.getStandardScore().compareTo(baseScore) > 0) {
                    throw new BusinessException("【" + indicatorDto.getName() + "】的阶梯标准分不能超过绩效分值");
                }
            }
            //校验阶梯
            CommonService.checkLadders(ladders, indicatorDto.getName());

            //校验条件
            for (EvalGroupIndicatorPreconditionDTO condDTO : indicatorDto.getConds()) {
                CommonService.checkLadders(condDTO.getLadders(), indicatorDto.getName());
            }

        }
    }

    /**
     * 检查阶梯分值
     * @param rewards
     */
    private void checkEvalRewardLadders(List<EvalGroupRewardDTO> rewards) {
        for (EvalGroupRewardDTO reward : rewards) {

            String rewardName = reward.getName();
            List<EvalGroupRewardLaddersDTO> ladders = reward.getLadders();
            if (EvalRewardCalMethodEnum.RANK.equals(reward.getCalMethod()) || EvalRewardCalMethodEnum.LADDER_DOUBLE.equals(reward.getCalMethod())) {
                BV.isTrue(PublicUtil.isNotEmpty(ladders), "阶梯不能为空");
                Collections.sort(ladders);
            }


            switch (reward.getCalMethod()) {
                case TOTAL_PROPORTION: case COMPLETE_RATIO: case ADDITIONAL_REWARDS:{
                    if (PublicUtil.isNotEmpty(ladders)) {
                        throw new BusinessException("计算方式【" + reward.getCalMethod().getName() + "】不能配置阶梯");
                    }
                    break;
                }
                case RANK: {
                    if (RankTypeEnum.NUMBER.equals(reward.getRankType())) {
                        checkLaddersStaffOrder(reward.getName(), ladders);
                        return;
                    }
                    CommonService.checkLadders(ladders, rewardName);
                    break;
                }
                case LADDER_DOUBLE: {
                    CommonService.checkLadders(ladders, rewardName);
                    break;
                }
            }

        }
    }

    /**
     * 检查按序号排名阶梯
     *
     * @param rewardName
     * @param ladders
     */
    public void checkLaddersStaffOrder(String rewardName, List<EvalGroupRewardLaddersDTO> ladders) {
        if (PublicUtil.isEmpty(ladders)) {
            throw new BusinessException("【" + rewardName + "】必须配置阶梯");
        }

        BigDecimal upper = null;
        for (EvalGroupRewardLaddersDTO ladder : ladders) {
            BigDecimal lower = ladder.getLower();
            if (PublicUtil.isNotEmpty(upper)) {
                BV.isTrue(lower.add(new BigDecimal(1)).compareTo(upper) == 0, "【" + rewardName + "】人员排名不是连续");
            }
            upper = ladder.getUpper();
        }
    }

    /**
     * 检查考评组指标
     *
     * @param evalGroupDTO
     */
    public void checkIndicator(EvalGroupDTO evalGroupDTO) {
        for (EvalGroupIndicatorDTO indicator : evalGroupDTO.getIndicators()) {
            if (PublicUtil.isNotEmpty(indicator.getLadderParams())) {
                BigDecimal proportion = indicator.getLadderParams().stream()
                        .map(EvalGroupIndicatorParamDTO::getProportion)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BV.isTrue(Constant.ONE_HUNDRED.compareTo(proportion) == 0, "【" + indicator.getName() + "】台阶占比总和必须为100");

                for (EvalGroupIndicatorParamDTO paramDTO : indicator.getLadderParams()) {
                    if (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()) &&
                            (PublicUtil.isEmpty(paramDTO.getTargetType()) || PublicUtil.isEmpty(paramDTO.getTargetValue()))) {
                        paramDTO.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }

            if (PublicUtil.isNotEmpty(indicator.getCommissionParams())) {
                BigDecimal proportion = indicator.getCommissionParams().stream()
                        .map(EvalGroupIndicatorParamDTO::getProportion)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BV.isTrue(Constant.ONE_HUNDRED.compareTo(proportion) == 0, "【" + indicator.getName() + "】提成占比总和必须为100");

                for (EvalGroupIndicatorParamDTO paramDTO : indicator.getCommissionParams()) {
                    if (! TargetTypeEnum.NO.equals(paramDTO.getTargetType()) &&
                            (PublicUtil.isEmpty(paramDTO.getTargetType()) || PublicUtil.isEmpty(paramDTO.getTargetValue()))) {
                        paramDTO.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }

            if (PublicUtil.isNotEmpty(indicator.getConds())) {
                for (EvalGroupIndicatorPreconditionDTO preconditionDTO : indicator.getConds()) {
                    if (! TargetTypeEnum.NO.equals(preconditionDTO.getTargetType()) &&
                            (PublicUtil.isEmpty(preconditionDTO.getTargetType()) || PublicUtil.isEmpty(preconditionDTO.getTargetValue()))) {
                        preconditionDTO.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }

        }
    }

    /**
     * 检查考评组指标
     *
     * @param evalGroupDTO
     */
    public void checkReward(EvalGroupDTO evalGroupDTO) {
        for (EvalGroupRewardDTO reward : evalGroupDTO.getRewards()) {
            String rewardName = reward.getName();

            if (PublicUtil.isNotEmpty(reward.getCommissionParams())) {
                BigDecimal proportion = reward.getCommissionParams().stream()
                        .map(EvalGroupRewardParamDTO::getProportion)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BV.isTrue(Constant.ONE_HUNDRED.compareTo(proportion) == 0, "【" + rewardName + "】台阶占比总和必须为100");

                for (EvalGroupRewardParamDTO commissionParam : reward.getCommissionParams()) {
                    if (! TargetTypeEnum.NO.equals(commissionParam.getTargetType()) &&
                            (PublicUtil.isEmpty(commissionParam.getTargetType()) || PublicUtil.isEmpty(commissionParam.getTargetValue()))) {
                        commissionParam.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }

            if (PublicUtil.isNotEmpty(reward.getLadderParams())) {
                BigDecimal proportion = reward.getLadderParams().stream()
                        .map(EvalGroupRewardParamDTO::getProportion)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                BV.isTrue(Constant.ONE_HUNDRED.compareTo(proportion) == 0, "【" + rewardName + "】提成占比总和必须为100");

                for (EvalGroupRewardParamDTO commissionParam : reward.getLadderParams()) {
                    if (! TargetTypeEnum.NO.equals(commissionParam.getTargetType()) &&
                            (PublicUtil.isEmpty(commissionParam.getTargetType()) || PublicUtil.isEmpty(commissionParam.getTargetValue()))) {
                        commissionParam.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }

            if (PublicUtil.isNotEmpty(reward.getConds())) {
                for (EvalGroupRewardPreconditionDTO preconditionDTO : reward.getConds()) {
                    if (! TargetTypeEnum.NO.equals(preconditionDTO.getTargetType()) &&
                            (PublicUtil.isEmpty(preconditionDTO.getTargetType()) || PublicUtil.isEmpty(preconditionDTO.getTargetValue()))) {
                        preconditionDTO.setTargetType(TargetTypeEnum.NO);
                    }
                }
            }
        }
    }

    /**
     * 考评组详情
     *
     * @param evalId
     * @return
     */
    public EvalVO evalDetail(Long evalId) {

        Eval eval = evalService.getById(evalId);
        BV.notNull(eval, "考评配置不存在，请重试");
        EvalVO evalVO = PublicUtil.copy(eval, EvalVO.class);

        List<EvalGroup> evalGroups = evalGroupService.list(Wrappers.<EvalGroup>lambdaQuery()
                .in(EvalGroup::getEvalId, evalId)
                .in(EvalGroup::getYn, Boolean.TRUE)
        );
        List<EvalGroupVO> evalGroupVOs = PublicUtil.copyList(evalGroups, EvalGroupVO.class);
        for (EvalGroupVO evalGroupVO : evalGroupVOs) {
            this.setEvalGroupIndicator(evalGroupVO);
            this.setEvalGroupReward(evalGroupVO);
        }
        evalVO.setEvalGroups(evalGroupVOs);

        return evalVO;
    }


    /**
     * 设置奖励
     *
     * @param evalGroupVO
     */
    public void setEvalGroupReward(EvalGroupVO evalGroupVO) {
        Long evalGroupId = evalGroupVO.getId();
        List<EvalGroupReward> rewards = evalGroupRewardService.list(Wrappers.<EvalGroupReward>lambdaQuery()
                .eq(EvalGroupReward::getEvalGroupId, evalGroupId)
                .eq(EvalGroupReward::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(rewards)) {
            List<EvalGroupRewardVO> rewardVOS = PublicUtil.copyList(rewards, EvalGroupRewardVO.class);
            evalGroupVO.setRewards(rewardVOS);

            Map<Long, List<EvalGroupRewardParamVO>> rewardParamMap = this.getEvalGroupRewardParamVO(evalGroupId);
            Map<Long, List<EvalGroupRewardPreconditionVO>> rewradCondMap = this.getEvalGroupRewardPreconditionVO(evalGroupId);

            for (EvalGroupRewardVO reward : rewardVOS) {
                Long rewardId = reward.getId();

                List<EvalGroupRewardParamVO> rewardParamVOS = rewardParamMap.getOrDefault(rewardId, new ArrayList<>());
                Map<ParamTypeEnum, List<EvalGroupRewardParamVO>> paramTypeMap = rewardParamVOS.stream()
                        .collect(Collectors.groupingBy(EvalGroupRewardParamVO::getParamType));
                reward.setCommissionParams(paramTypeMap.getOrDefault(ParamTypeEnum.COMMISSION, new ArrayList<>()));
                reward.setLadderParams(paramTypeMap.getOrDefault(ParamTypeEnum.LADDER, new ArrayList<>()));
                reward.setConds(rewradCondMap.getOrDefault(rewardId, new ArrayList<>()));


                List<EvalGroupRewardLadders> rewardLadders = evalGroupRewardLaddersService.list(Wrappers.<EvalGroupRewardLadders>lambdaQuery()
                        .eq(EvalGroupRewardLadders::getEvalGroupId, evalGroupId)
                        .eq(EvalGroupRewardLadders::getYn, Boolean.TRUE)
                );
                List<EvalGroupRewardLaddersVO> rewardLaddersVOS = new ArrayList<>();
                for (EvalGroupRewardLadders rewardLadder : rewardLadders) {
                    EvalGroupRewardLaddersVO rewardLaddersVO = PublicUtil.copy(rewardLadder, EvalGroupRewardLaddersVO.class);
                    if (EvalRewardCalMethodEnum.RANK.equals(reward.getCalMethod()) && RankTypeEnum.PERCENT.equals(reward.getRankType())) {
                        rewardLadder.setUpper(rewardLadder.getUpper().multiply(Constant.ONE_HUNDRED));
                        rewardLadder.setLower(rewardLadder.getLower().multiply(Constant.ONE_HUNDRED));
                    }
                    rewardLaddersVOS.add(rewardLaddersVO);
                }
                reward.setLadders(rewardLaddersVOS);

                setRewardTargetVos(reward);
            }
        }
    }

    /**
     * 装换为目标对象
     *
     * @return
     */
    public void setRewardTargetVos(EvalGroupRewardVO reward) {

        List<EvalGroupRewardParamVO> ladderParams = reward.getLadderParams();
        List<EvalGroupRewardParamVO> commissionParams = reward.getCommissionParams();
        List<EvalGroupRewardPreconditionVO> conds = reward.getConds();

        List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
        for (EvalGroupRewardParamVO paramVO : ladderParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupRewardParamVO paramVO : commissionParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupRewardPreconditionVO preconditionVO : conds) {
            if (PublicUtil.isNotEmpty(preconditionVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        reward.setTargets(targetVOS);
    }
    /**
     * 装换为目标对象
     *
     * @return
     */
    public void setIndicatorTargetVos(EvalGroupIndicatorVO indicatorVO) {

        List<EvalGroupIndicatorParamVO> ladderParams = indicatorVO.getLadderParams();
        List<EvalGroupIndicatorParamVO> commissionParams = indicatorVO.getCommissionParams();
        List<EvalGroupIndicatorPreconditionVO> conds = indicatorVO.getConds();

        List<EvalGroupIndicatorTargetVO> targetVOS = new ArrayList<>();
        for (EvalGroupIndicatorParamVO paramVO : ladderParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupIndicatorParamVO paramVO : commissionParams) {
            if (PublicUtil.isNotEmpty(paramVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(paramVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        for (EvalGroupIndicatorPreconditionVO preconditionVO : conds) {
            if (PublicUtil.isNotEmpty(preconditionVO.getTargetType())) {
                EvalGroupIndicatorTargetVO targetVO = PublicUtil.copy(preconditionVO, EvalGroupIndicatorTargetVO.class);
                if (TargetTypeEnum.RATIO.equals(targetVO.getTargetType())) {
                    targetVO.setTargetValue(targetVO.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                targetVOS.add(targetVO);
            }
        }

        indicatorVO.setTargets(targetVOS);
    }


    /**
     * 考评奖惩条件
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupRewardPreconditionVO>> getEvalGroupRewardPreconditionVO(Long evalGroupId) {
        List<EvalGroupRewardPrecondition> rewardPreconditions = evalGroupRewardPreconditionService.list(Wrappers.<EvalGroupRewardPrecondition>lambdaQuery()
                .eq(EvalGroupRewardPrecondition::getEvalGroupId, evalGroupId)
                .eq(EvalGroupRewardPrecondition::getYn, Boolean.TRUE)
        );

        List<EvalGroupRewardPreconditionVO> rewardPreconditionVOS = new ArrayList<>();
        for (EvalGroupRewardPrecondition precondition : rewardPreconditions) {
            EvalGroupRewardPreconditionVO preconditionVO = PublicUtil.copy(precondition, EvalGroupRewardPreconditionVO.class);
            if (TargetTypeEnum.RATIO.equals(preconditionVO.getTargetType())) {
                preconditionVO.setTargetValue(preconditionVO.getTargetValue().multiply(Constant.ONE_HUNDRED));
            }
            rewardPreconditionVOS.add(preconditionVO);
        }

        Map<Long, List<EvalGroupRewardPreconditionVO>> rewardCondMap = rewardPreconditionVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupRewardPreconditionVO::getEvalGroupRewardId));

        return rewardCondMap;
    }

    /**
     * 考评奖惩参数
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupRewardParamVO>> getEvalGroupRewardParamVO(Long evalGroupId) {
        List<EvalGroupRewardParam> rewardParams = evalGroupRewardParamService.list(Wrappers.<EvalGroupRewardParam>lambdaQuery()
                .eq(EvalGroupRewardParam::getEvalGroupId, evalGroupId)
                .eq(EvalGroupRewardParam::getYn, Boolean.TRUE)
        );

        List<EvalGroupRewardParamVO> rewardParamVOS = new ArrayList<>();
        for (EvalGroupRewardParam rewardParam : rewardParams) {
            EvalGroupRewardParamVO paramVO = PublicUtil.copy(rewardParam, EvalGroupRewardParamVO.class);
            if (TargetTypeEnum.RATIO.equals(paramVO.getTargetType())) {
                paramVO.setTargetValue(paramVO.getTargetValue().multiply(Constant.ONE_HUNDRED));
            }
            rewardParamVOS.add(paramVO);
        }

        Map<Long, List<EvalGroupRewardParamVO>> rewardParamMap = rewardParamVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupRewardParamVO::getEvalGroupRewardId));

        return rewardParamMap;
    }

    /**
     * 设置考评指标
     *
     * @param evalGroupVO
     */
    public void setEvalGroupIndicator(EvalGroupVO evalGroupVO) {
        Long evalGroupId = evalGroupVO.getId();
        List<EvalGroupIndicator> indicators = evalGroupIndicatorService.list(Wrappers.<EvalGroupIndicator>lambdaQuery()
                .eq(EvalGroupIndicator::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicator::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isNotEmpty(indicators)) {
            List<EvalGroupIndicatorVO> indicatorVOS = PublicUtil.copyList(indicators, EvalGroupIndicatorVO.class);
            evalGroupVO.setIndicators(indicatorVOS);

            Map<Long, List<EvalGroupIndicatorParamVO>> paramMap = this.getEvalGroupIndicatorParamVO(evalGroupId);
            Map<Long, List<EvalGroupIndicatorPreconditionVO>> condMap = this.getEvalGroupIndicatorCondVO(evalGroupId);

            for (EvalGroupIndicatorVO indicatorVO : indicatorVOS) {
                Long kpiGroupIndicatorId = indicatorVO.getId();

                //参数
                List<EvalGroupIndicatorParamVO> indicatorParamVOS = paramMap.getOrDefault(kpiGroupIndicatorId, new ArrayList<>());
                Map<ParamTypeEnum, List<EvalGroupIndicatorParamVO>> paramTypeMap = indicatorParamVOS.stream()
                        .collect(Collectors.groupingBy(EvalGroupIndicatorParamVO::getParamType));
                indicatorVO.setCommissionParams(paramTypeMap.getOrDefault(ParamTypeEnum.COMMISSION, new ArrayList<>()));
                indicatorVO.setLadderParams(paramTypeMap.getOrDefault(ParamTypeEnum.LADDER, new ArrayList<>()));

                //条件
                List<EvalGroupIndicatorPreconditionVO> preconditionVOS = condMap.getOrDefault(kpiGroupIndicatorId, new ArrayList<>());
                for (EvalGroupIndicatorPreconditionVO cond : preconditionVOS) {
                    TargetTypeEnum targetType = cond.getTargetType();
                    List<EvalGroupIndicatorPreconditionLadders> condLadders = evalGroupIndicatorPreconditionLaddersService
                            .list(Wrappers.<EvalGroupIndicatorPreconditionLadders>lambdaQuery()
                                    .eq(EvalGroupIndicatorPreconditionLadders::getPreconditionId, cond.getId())
                                    .eq(EvalGroupIndicatorPreconditionLadders::getYn, Boolean.TRUE)
                            );

                    List<EvalGroupIndicatorPreconditionLaddersVO> condLaddersVOS = new ArrayList<>();
                    for (EvalGroupIndicatorPreconditionLadders condLadder : condLadders) {
                        EvalGroupIndicatorPreconditionLaddersVO condLaddersVO = PublicUtil.copy(condLadder, EvalGroupIndicatorPreconditionLaddersVO.class);
                        if (TargetTypeEnum.RATIO.equals(targetType)) {
                            condLaddersVO.convertToPercent();
                        }
                        condLaddersVOS.add(condLaddersVO);
                    }
                    cond.setLadders(condLaddersVOS);
                }
                Collections.sort(preconditionVOS, Comparator.comparingInt(EvalGroupIndicatorPreconditionVO::getSort));
                indicatorVO.setConds(preconditionVOS);

                setIndicatorTargetVos(indicatorVO);
            }
        }
    }

    /**
     * 考评参数
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupIndicatorParamVO>> getEvalGroupIndicatorParamVO(Long evalGroupId) {
        List<EvalGroupIndicatorParam> params = evalGroupIndicatorParamService.list(Wrappers.<EvalGroupIndicatorParam>lambdaQuery()
                .eq(EvalGroupIndicatorParam::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicatorParam::getYn, Boolean.TRUE)
        );

        List<EvalGroupIndicatorParamVO> paramVOS = new ArrayList<>();
        for (EvalGroupIndicatorParam param : params) {
            EvalGroupIndicatorParamVO paramVO = PublicUtil.copy(param, EvalGroupIndicatorParamVO.class);
            if (TargetTypeEnum.RATIO.equals(paramVO.getTargetType())) {
                paramVO.setTargetValue(paramVO.getTargetValue().multiply(Constant.ONE_HUNDRED));
            }
            paramVOS.add(paramVO);
        }

        Map<Long, List<EvalGroupIndicatorParamVO>> indicatorParamMap = paramVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupIndicatorParamVO::getEvalGroupIndicatorId));

        return indicatorParamMap;
    }

    /**
     * 获取指标条件
     *
     * @param evalGroupId
     * @return
     */
    public Map<Long, List<EvalGroupIndicatorPreconditionVO>> getEvalGroupIndicatorCondVO(Long evalGroupId) {
        List<EvalGroupIndicatorPrecondition> preconditions = evalGroupIndicatorPreconditionService.list(Wrappers.<EvalGroupIndicatorPrecondition>lambdaQuery()
                .eq(EvalGroupIndicatorPrecondition::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicatorPrecondition::getYn, Boolean.TRUE)
        );

        List<EvalGroupIndicatorPreconditionVO> preconditionVOS = new ArrayList<>();
        for (EvalGroupIndicatorPrecondition precondition : preconditions) {
            EvalGroupIndicatorPreconditionVO preconditionVO = PublicUtil.copy(precondition, EvalGroupIndicatorPreconditionVO.class);
            if (TargetTypeEnum.RATIO.equals(preconditionVO.getTargetType())) {
                preconditionVO.setTargetValue(preconditionVO.getTargetValue().multiply(Constant.ONE_HUNDRED));
            }
            preconditionVOS.add(preconditionVO);
        }

        Map<Long, List<EvalGroupIndicatorPreconditionVO>> indicatorCondMap = preconditionVOS.stream()
                .collect(Collectors.groupingBy(EvalGroupIndicatorPreconditionVO::getEvalGroupIndicatorId));

        return indicatorCondMap;
    }




}
