package cn.fw.morax.service.biz.eval;

import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.ApprovalResultEvent;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.SettingDraft;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.dto.eval.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.SettingDraftService;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.RoundingMode;
import java.util.*;

import static cn.fw.common.businessvalidator.Validator.BV;

@Service
@Slf4j
@RequiredArgsConstructor
public class EvalGroupDataService {

    private final EvalGroupIndicatorPreconditionLaddersService evalGroupIndicatorPreconditionLaddersService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final EvalGroupIndicatorLaddersService evalGroupIndicatorLaddersService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupRewardLaddersService evalGroupRewardLaddersService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalIndicatorService evalIndicatorService;
    private final ApprovalRecordService approvalRecordService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final EvalGroupService evalGroupService;
    private final FlowApproveRpc flowApproveRpc;
    private final EvalService evalService;
    private final SettingDraftService settingDraftService;
    private final FlowVal flowVal;

    /**
     * 保存绩效组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveSettingDraft(EvalDTO dto, Eval eval) {
        SettingDraft settingDraft = null;
        if (PublicUtil.isNotEmpty(dto.getId())) {
            settingDraft = settingDraftService.getOne(Wrappers.<SettingDraft>lambdaQuery()
                    .eq(SettingDraft::getUnionId, dto.getId())
                    .orderByDesc(SettingDraft::getId)
            );
        }
        if (PublicUtil.isEmpty(settingDraft)) {
            settingDraft = new SettingDraft();
            settingDraft.setUnionId(eval.getId());
            settingDraft.setType(SettingDraftTypeEnum.EVAL);
        }
        settingDraft.setContent(JSON.toJSONString(dto));
        settingDraftService.save(settingDraft);
    }

    /**
     * 保存绩效组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public Eval saveEval(EvalDTO dto) {
        Eval eval = PublicUtil.copy(dto, Eval.class);
        eval.setStatus(SettingStatusEnum.APPROVING);
        evalService.saveOrUpdate(eval);
        return eval;
    }

    /**
     * 保存绩效组
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public EvalGroup saveEvalGroup(EvalGroupDTO dto, Eval eval) {
        EvalGroup evalGroup = PublicUtil.copy(dto, EvalGroup.class);
        evalGroup.setIndicatorNum(dto.getIndicators().size());
        if (PublicUtil.isNotEmpty(dto.getId())) {
            evalGroup.setEgc(dto.getEgc());
        } else {
            evalGroup.setEgc(PublicUtil.getUUID());
        }
        evalGroup.setEvalId(eval.getId());
//        evalGroup.setStatus(SettingStatusEnum.APPROVING);
        evalGroupService.save(evalGroup);
        return evalGroup;
    }

    /**
     * 保存考评组指标
     *
     * @param evalGroup
     * @param indicatorDTOS
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalGroupIndicator(EvalGroup evalGroup, List<EvalGroupIndicatorDTO> indicatorDTOS) {
        LinkedList<EvalGroupIndicatorParam> indicatorParams = Lists.newLinkedList();
        LinkedList<EvalGroupIndicatorPreconditionLadders> preconditionLadders = Lists.newLinkedList();
        LinkedList<EvalGroupIndicatorLadders> indicatorLadders = Lists.newLinkedList();
        Long evalGroupId = evalGroup.getId();
        Long indicatorId = null;
        for (EvalGroupIndicatorDTO indicatorDto : indicatorDTOS) {
            EvalGroupIndicator indicator = PublicUtil.copy(indicatorDto, EvalGroupIndicator.class);
            indicator.setEvalGroupId(evalGroupId);

            evalGroupIndicatorService.save(indicator);
            indicatorId = indicator.getId();

            //参数
            for (EvalGroupIndicatorParamDTO paramDTO : Optional.ofNullable(indicatorDto.getCommissionParams()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorParam param = PublicUtil.copy(paramDTO, EvalGroupIndicatorParam.class);
                param.setEvalGroupId(evalGroupId);
                param.setEvalGroupIndicatorId(indicatorId);
                param.setParamType(ParamTypeEnum.COMMISSION);
                if (PublicUtil.isNotEmpty(param.getTargetType()) && TargetTypeEnum.RATIO.equals(param.getTargetType())) {
                    param.setTargetValue(param.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                indicatorParams.add(param);
            }

            for (EvalGroupIndicatorParamDTO paramDTO : Optional.ofNullable(indicatorDto.getLadderParams()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorParam param = PublicUtil.copy(paramDTO, EvalGroupIndicatorParam.class);
                param.setEvalGroupId(evalGroupId);
                param.setEvalGroupIndicatorId(indicatorId);
                param.setParamType(ParamTypeEnum.LADDER);
                if (PublicUtil.isNotEmpty(param.getTargetType()) && TargetTypeEnum.RATIO.equals(param.getTargetType())) {
                    param.setTargetValue(param.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                indicatorParams.add(param);
            }

            for (EvalGroupIndicatorLaddersDTO laddersDTO : Optional.ofNullable(indicatorDto.getLadders()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorLadders ladder = PublicUtil.copy(laddersDTO, EvalGroupIndicatorLadders.class);
                ladder.setEvalGroupIndicatorId(indicatorId);
                ladder.setUpper(ladder.getUpper().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                ladder.setLower(ladder.getLower().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                indicatorLadders.add(ladder);
            }

            for (EvalGroupIndicatorPreconditionDTO preconditionDTO : Optional.ofNullable(indicatorDto.getConds()).orElse(new ArrayList<>())) {
                EvalGroupIndicatorPrecondition cond = PublicUtil.copy(preconditionDTO, EvalGroupIndicatorPrecondition.class);
                cond.setEvalGroupId(evalGroupId);
                cond.setEvalGroupIndicatorId(indicatorId);
                if (PublicUtil.isNotEmpty(cond.getTargetType()) && TargetTypeEnum.RATIO.equals(cond.getTargetType())) {
                    cond.setTargetValue(cond.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                evalGroupIndicatorPreconditionService.save(cond);
                for (EvalGroupIndicatorPreconditionLaddersDTO preconditionLaddersDTO : preconditionDTO.getLadders()) {
                    EvalGroupIndicatorPreconditionLadders condLadder = PublicUtil.copy(preconditionLaddersDTO, EvalGroupIndicatorPreconditionLadders.class);
                    condLadder.setPreconditionId(cond.getId());
                    condLadder.setUpper(condLadder.getUpper().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                    condLadder.setLower(condLadder.getLower().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                    preconditionLadders.add(condLadder);
                }
            }

        }

        if (PublicUtil.isNotEmpty(indicatorParams)) {
            evalGroupIndicatorParamService.saveBatch(indicatorParams);
        }
        if (PublicUtil.isNotEmpty(preconditionLadders)) {
            evalGroupIndicatorPreconditionLaddersService.saveBatch(preconditionLadders);
        }
        if (PublicUtil.isNotEmpty(indicatorLadders)) {
            evalGroupIndicatorLaddersService.saveBatch(indicatorLadders);
        }
    }

    /**
     * 保存考评组奖励
     *
     * @param evalGroup
     * @param rewardDTOS
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalGroupReward(EvalGroup evalGroup, List<EvalGroupRewardDTO> rewardDTOS) {
        Long evalGroupId = evalGroup.getId();
        LinkedList<EvalGroupRewardParam> rewardParams = Lists.newLinkedList();
        LinkedList<EvalGroupRewardLadders> rewardLadders = Lists.newLinkedList();
        LinkedList<EvalGroupRewardPrecondition> rewardPreconditions = Lists.newLinkedList();
        for (EvalGroupRewardDTO rewardDTO : rewardDTOS) {
            EvalGroupReward reward = PublicUtil.copy(rewardDTO, EvalGroupReward.class);
            reward.setEvalGroupId(evalGroupId);
            evalGroupRewardService.save(reward);
            Long rewardId = reward.getId();

            for (EvalGroupRewardParamDTO rewardParamDTO : Optional.ofNullable(rewardDTO.getLadderParams()).orElse(new ArrayList<>())) {
                EvalGroupRewardParam rewardParam = PublicUtil.copy(rewardParamDTO, EvalGroupRewardParam.class);
                rewardParam.setEvalGroupId(evalGroupId);
                rewardParam.setEvalGroupRewardId(rewardId);
                rewardParam.setParamType(ParamTypeEnum.LADDER);
                if (PublicUtil.isNotEmpty(rewardParam.getTargetType()) && TargetTypeEnum.RATIO.equals(rewardParam.getTargetType())) {
                    rewardParam.setTargetValue(rewardParam.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                rewardParams.add(rewardParam);
            }

            for (EvalGroupRewardParamDTO rewardParamDTO : Optional.ofNullable(rewardDTO.getCommissionParams()).orElse(new ArrayList<>())) {
                EvalGroupRewardParam rewardParam = PublicUtil.copy(rewardParamDTO, EvalGroupRewardParam.class);
                rewardParam.setEvalGroupId(evalGroupId);
                rewardParam.setEvalGroupRewardId(rewardId);
                rewardParam.setParamType(ParamTypeEnum.COMMISSION);
                if (PublicUtil.isNotEmpty(rewardParam.getTargetType()) && TargetTypeEnum.RATIO.equals(rewardParam.getTargetType())) {
                    rewardParam.setTargetValue(rewardParam.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                rewardParams.add(rewardParam);
            }

            for (EvalGroupRewardLaddersDTO rewardLaddersDTO : Optional.ofNullable(rewardDTO.getLadders()).orElse(new ArrayList<>())) {
                EvalGroupRewardLadders rewardLadder = this.convertRewardLaddersDB(rewardDTO, rewardLaddersDTO);
                rewardLadder.setEvalGroupId(evalGroupId);
                rewardLadder.setEvalGroupRewardId(rewardId);
                rewardLadders.add(rewardLadder);
            }

            for (EvalGroupRewardPreconditionDTO rewardPreconditionDTO : Optional.ofNullable(rewardDTO.getConds()).orElse(new ArrayList<>())) {
                EvalGroupRewardPrecondition rewardPrecondition = PublicUtil.copy(rewardPreconditionDTO, EvalGroupRewardPrecondition.class);
                rewardPrecondition.setEvalGroupId(evalGroupId);
                rewardPrecondition.setEvalGroupRewardId(rewardId);
                if (PublicUtil.isNotEmpty(rewardPrecondition.getTargetType()) && TargetTypeEnum.RATIO.equals(rewardPrecondition.getTargetType())) {
                    rewardPrecondition.setTargetValue(rewardPrecondition.getTargetValue().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
                }
                rewardPreconditions.add(rewardPrecondition);
            }
        }

        if (PublicUtil.isNotEmpty(rewardParams)) {
            evalGroupRewardParamService.saveBatch(rewardParams);
        }
        if (PublicUtil.isNotEmpty(rewardLadders)) {
            evalGroupRewardLaddersService.saveBatch(rewardLadders);
        }
        if (PublicUtil.isNotEmpty(rewardPreconditions)) {
            evalGroupRewardPreconditionService.saveBatch(rewardPreconditions);
        }
    }


    /**
     * 转换
     *
     * @param rewardDTO
     * @param rewardLaddersDTO
     * @return
     */
    public EvalGroupRewardLadders convertRewardLaddersDB(EvalGroupRewardDTO rewardDTO, EvalGroupRewardLaddersDTO rewardLaddersDTO) {
        EvalGroupRewardLadders rewardLadder = PublicUtil.copy(rewardLaddersDTO, EvalGroupRewardLadders.class);
        if (EvalRewardCalMethodEnum.RANK.equals(rewardDTO.getCalMethod()) && RankTypeEnum.NUMBER.equals(rewardDTO.getRankType())) {
            return rewardLadder;
        }
        rewardLadder.setUpper(rewardLadder.getUpper().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
        rewardLadder.setLower(rewardLadder.getLower().divide(Constant.ONE_HUNDRED, 4, RoundingMode.HALF_UP));
        return rewardLadder;
    }

    /**
     * 审批考评组
     *
     * @param approvalRecord
     * @param result
     */
    public void approvalEvalConfig(ApprovalRecord approvalRecord, ApprovalResultEvent result) {
        log.info("收到岗位最低收入补偿审批信息：{}", JSON.toJSONString(result));
        Boolean agree = result.getAgree();
        Long compensationId = approvalRecord.getDataId();
        evalService.update(Wrappers.<Eval>lambdaUpdate()
                .set(Eval::getStatus, (agree) ? SettingStatusEnum.EFFECTIVE : SettingStatusEnum.INEFFECTIVE)
                .set(Eval::getUpdateTime, new Date())
                .eq(Eval::getId, compensationId)
        );
        if (! result.getAgree()) {
            return;
        }

    }

    /**
     * 删除考评
     *
     * @param id
     */
    public void delEval(Long id) {
        Eval eval = evalService.getById(id);
        BV.isTrue(PublicUtil.isNotEmpty(eval), "考评数据不存在");
        BV.isFalse(SettingStatusEnum.APPROVING.equals(eval.getStatus()), "正在审批中的考评不能删除");

        evalService.removeById(id);
        evalGroupService.update(Wrappers.<EvalGroup>lambdaUpdate()
                .set(EvalGroup::getYn, Boolean.FALSE)
                .set(EvalGroup::getUpdateTime, new Date())
                .eq(EvalGroup::getEvalId, id)
        );

    }

    /**
     * 获取考评草稿
     *
     * @param id
     * @return
     */
    public EvalDTO getEvalDraft(Long id) {
        SettingDraft settingDraft = settingDraftService.getOne(Wrappers.<SettingDraft>lambdaQuery()
                .eq(SettingDraft::getUnionId, id)
                .eq(SettingDraft::getYn, Boolean.TRUE)
        );

        BV.isTrue(PublicUtil.isNotEmpty(settingDraft), "考评草稿不存在");
        EvalDTO evalDTO = JSON.parseObject(settingDraft.getContent(), EvalDTO.class);
        return evalDTO;
    }

    /**
     * 保存考评草稿
     *
     * @param dto
     * @return
     */
    public void saveEvalDraft(EvalDTO dto) {
        Eval eval = null;
        SettingDraft settingDraft = null;
        if (PublicUtil.isNotEmpty(dto.getId())) {
            eval = evalService.getById(dto.getId());
            BV.isTrue(PublicUtil.isNotEmpty(eval), "考评数据不存在");

            settingDraft = settingDraftService.getOne(Wrappers.<SettingDraft>lambdaQuery()
                            .eq(SettingDraft::getUnionId, dto.getId())
                            .eq(SettingDraft::getYn, Boolean.TRUE)
                    , Boolean.FALSE);
            BV.isTrue(PublicUtil.isNotEmpty(settingDraft), "考评草稿数据不存在");
        } else {
            eval.setStatus(SettingStatusEnum.DRAFT);
            eval = PublicUtil.copy(dto, Eval.class);

            settingDraft = new SettingDraft();
            settingDraft.setUnionId(eval.getId());
            settingDraft.setType(SettingDraftTypeEnum.EVAL);
        }
        evalService.saveOrUpdate(eval);

        settingDraft.setContent(JSON.toJSONString(dto));
        settingDraftService.saveOrUpdate(settingDraft);
    }
}
