package cn.fw.morax.service.biz.eval;

import cn.fw.approval.sdk.param.InitiateApprovalParam;
import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.config.FlowVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.pojo.event.ApprovalResultEvent;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.dao.eval.EvalGroupIndicatorParamDao;
import cn.fw.morax.dao.eval.EvalGroupIndicatorPreconditionDao;
import cn.fw.morax.dao.eval.EvalGroupRewardDao;
import cn.fw.morax.dao.eval.EvalGroupRewardPreconditionDao;
import cn.fw.morax.domain.db.ApprovalRecord;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.db.kpi.Indicators;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicator;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorParam;
import cn.fw.morax.domain.db.salary.IncomeCompensation;
import cn.fw.morax.domain.dto.EvalIndicatorDTO;
import cn.fw.morax.domain.dto.IncomeCompensationDTO;
import cn.fw.morax.domain.dto.eval.*;
import cn.fw.morax.domain.dto.kpi.KpiGroupDTO;
import cn.fw.morax.domain.dto.query.EvalIndicatorQueryDTO;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.eval.EvalIndicatorVO;
import cn.fw.morax.domain.vo.salary.IncomeCompensationVO;
import cn.fw.morax.rpc.approval.FlowApproveRpc;
import cn.fw.morax.rpc.approval.dto.FlowDto;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.data.ApprovalRecordService;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Valid;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@Service
@Slf4j
@RequiredArgsConstructor
public class EvalIndicatorBizService {

    private final EvalIndicatorService evalIndicatorService;
    private final ApprovalRecordService approvalRecordService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final EvalGroupService evalGroupService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalGroupIndicatorParamService evalGroupIndicatorParamService;
    private final EvalGroupIndicatorPreconditionService evalGroupIndicatorPreconditionService;
    private final EvalGroupIndicatorPreconditionLaddersService evalGroupIndicatorPreconditionLaddersService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalGroupRewardParamService evalGroupRewardParamService;
    private final EvalGroupRewardPreconditionService evalGroupRewardPreconditionService;
    private final FlowApproveRpc flowApproveRpc;
    private final FlowVal flowVal;

    @Value("${spring.cache.custom.global-prefix}:eval:indicator:save:")
    @Getter
    private String savePrefix;
    @Value("${spring.cache.custom.global-prefix}:eval:indicator:enable:")
    @Getter
    private String enablePrefix;
    @Value("${spring.cache.custom.global-prefix}:eval:group:save:")
    @Getter
    private String saveGroupPrefix;

    /**
     * 分页查询
     *
     * @param dto
     * @return
     */
    public AppPage<EvalIndicatorVO> evalIndicatorPage(EvalIndicatorQueryDTO dto) {
        PageData<EvalIndicator> pageData = evalIndicatorService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<EvalIndicator>lambdaQuery()
                        .eq(PublicUtil.isNotEmpty(dto.getSysId()), EvalIndicator::getSysId, dto.getSysId())
                        .eq(PublicUtil.isNotEmpty(dto.getRoleCode()), EvalIndicator::getRoleCode, dto.getRoleCode())
                        .eq(PublicUtil.isNotEmpty(dto.getEnable()), EvalIndicator::getEnable, dto.getEnable())
                        .and(PublicUtil.isNotEmpty(dto.getKeyword()), w ->
                                w.like(EvalIndicator::getCode, dto.getKeyword()).or().like(EvalIndicator::getName, dto.getKeyword())
                        )
                        .eq(EvalIndicator::getYn, Boolean.TRUE)
        );
        return PublicUtil.toPage(pageData, evalIndicator -> {
            EvalIndicatorVO evalIndicatorVO = PublicUtil.copy(evalIndicator, EvalIndicatorVO.class);
            return evalIndicatorVO;
        });
    }

    /**
     * 考评指标详情
     *
     * @param id
     * @return
     */
    public EvalIndicatorVO evalIndicatorDetail(Long id) {
        EvalIndicator evalIndicator = evalIndicatorService.getById(id);
        return PublicUtil.copy(evalIndicator, EvalIndicatorVO.class);
    }

    /**
     * 启用禁用指标
     *
     * @param indicatorId
     * @param enable
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getEnablePrefix()", key = "#indicatorId", message = "修改中，请勿重复操作")
    public void enableIndicator(Long indicatorId, Boolean enable) {
        EvalIndicator evalIndicator = evalIndicatorService.getById(indicatorId);
        BV.notNull(evalIndicator, "指标不存在，请重试");
        BV.isFalse(enable.equals(evalIndicator.getEnable()), "指标已经是" + ((enable) ? "启用" : "禁用") + "状态，请重试");
        //禁用情况：正在使用的指标不能禁用，并提示哪些地方在用
//        if (! enable) {
//            List<KpiGroup> kpiGroups = evalIndicatorService.getEvalGroupByIndicatorId(indicatorId);
//            Set<String> postNames = kpiGroups.stream().map(KpiGroup::getPostName).collect(Collectors.toSet());
//            if (PublicUtil.isNotEmpty(postNames)) {
//                throw new BusinessException(String.join(",", postNames) + "岗位正在使用此绩效指标，不能禁用");
//            }
//        }
        evalIndicatorService.update(Wrappers.<EvalIndicator>lambdaUpdate()
                .set(EvalIndicator::getEnable, enable)
                .set(EvalIndicator::getUpdateTime, new Date())
                .eq(EvalIndicator::getId, indicatorId)
        );
    }

    /**
     * 保存
     *
     * @param dto
     * @param user
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getIndicatorName()", message = "保存中，请勿重复操作")
    public void saveEvalIndicator(EvalIndicatorDTO dto, LoginAuthBean user) {
        checkDto(dto);
        checkIndicatorName(dto.getId(), dto.getName());
        EvalIndicator evalIndicator = dto.convertToDB();

        if (PublicUtil.isEmpty(dto.getId())) {
            String indicatorCode = PublicUtil.getUUID();
            this.checkIndicatorCode(indicatorCode);
            evalIndicator.setCode(indicatorCode);
        } else {
            EvalIndicator originEvalIndicator = evalIndicatorService.getById(dto.getId());
            BV.notNull(evalIndicator, "指标不存在，请重试");
            evalIndicator.setCode(originEvalIndicator.getCode());
            evalIndicator.setId(originEvalIndicator.getId());
        }
        evalIndicatorService.saveOrUpdate(evalIndicator);

    }

    /**
     * 检查指标编码
     *
     * @param code
     */
    private void checkIndicatorCode(String code) {
        long taskCount = evalIndicatorService.count(Wrappers.<EvalIndicator>lambdaQuery()
                .eq(EvalIndicator::getCode, code)
        );
        if (taskCount > 0) {
            log.error("指标编码生成重复：{}", code);
            throw new BusinessException("指标编码已存在，请重试");
        }
    }

    /**
     * 检查指标名称
     *
     * @param indicatorName
     */
    private void checkIndicatorName(Long id, String indicatorName) {
        long taskCount = evalIndicatorService.count(Wrappers.<EvalIndicator>lambdaQuery()
                .eq(EvalIndicator::getName, indicatorName)
                .eq(EvalIndicator::getYn, Boolean.TRUE)
                .ne(PublicUtil.isNotEmpty(id), EvalIndicator::getId, id)
        );
        BV.isFalse(taskCount > 0, "指标名称已存在，请重新输入");
    }

    /**
     * 检查指标参数
     *
     * @param dto
     */
    private void checkDto(EvalIndicatorDTO dto) {

        Boolean paramError;
        switch (dto.getRuleType()) {
            case DAY:
                if (PublicUtil.isEmpty(dto.getStartTime()) || PublicUtil.isEmpty(dto.getRollValue())) {
                    throw new BusinessException("开始日期、滚动值不能为空");
                }
                break;
            case WEEK_DAY:
                if (PublicUtil.isEmpty(dto.getRuleValues())) {
                    throw new BusinessException("指定星期不能为空");
                }
                paramError = dto.getRuleValues().stream().filter(value -> (value > 7 || value < 1)).findFirst().isPresent();
                BV.isFalse(paramError, "指定星期参数异常");
                break;
            case MONTH_FIX_DAY:
                if (PublicUtil.isEmpty(dto.getRuleValues())) {
                    throw new BusinessException("指定日期不能为空");
                }
                paramError = dto.getRuleValues().stream().filter(value -> (value > 28 || value < 1)).findFirst().isPresent();
                BV.isFalse(paramError, "按月指定日期参数异常");
                break;
            case MONTH:
                //滚动月数、开始月数、滚动月开始日、滚动开始类型
                if (PublicUtil.isEmpty(dto.getMonthDayType()) || PublicUtil.isEmpty(dto.getMonthDay()) ||
                        PublicUtil.isEmpty(dto.getRollValue()) || PublicUtil.isEmpty(dto.getStartTime())) {
                    throw new BusinessException("开始日期、滚动值不能为空");
                }
                if (dto.getMonthDay() > 28 || dto.getMonthDay() < 1) {
                    throw new BusinessException("滚动月开始日错误");
                }
                break;
        }

    }
}